{-# OPTIONS_GHC -fno-warn-unused-binds -fno-warn-missing-signatures #-}
{-# LANGUAGE CPP #-}
{-# LINE 1 "LexIntel.x" #-}
module LexIntel (AlexPosn(..), alexScanTokens) where

import Intel (Reg32(..),Register(..))
import TokenIntel as Token
#if __GLASGOW_HASKELL__ >= 603
#include "ghcconfig.h"
#elif defined(__GLASGOW_HASKELL__)
#include "config.h"
#endif
#if __GLASGOW_HASKELL__ >= 503
import Data.Array
#else
import Array
#endif
#define ALEX_POSN 1
-- -----------------------------------------------------------------------------
-- Alex wrapper code.
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.

#if defined(ALEX_MONAD) || defined(ALEX_MONAD_BYTESTRING)
import Control.Applicative as App (Applicative (..))
#endif

import Data.Word (Word8)
#if defined(ALEX_BASIC_BYTESTRING) || defined(ALEX_POSN_BYTESTRING) || defined(ALEX_MONAD_BYTESTRING)

import Data.Int (Int64)
import qualified Data.Char
import qualified Data.ByteString.Lazy     as ByteString
import qualified Data.ByteString.Internal as ByteString (w2c)

#elif defined(ALEX_STRICT_BYTESTRING)

import qualified Data.Char
import qualified Data.ByteString          as ByteString
import qualified Data.ByteString.Internal as ByteString hiding (ByteString)
import qualified Data.ByteString.Unsafe   as ByteString

#else

import Data.Char (ord)
import qualified Data.Bits

-- | Encode a Haskell String to a list of Word8 values, in UTF8 format.
utf8Encode :: Char -> [Word8]
utf8Encode = uncurry (:) . utf8Encode'

utf8Encode' :: Char -> (Word8, [Word8])
utf8Encode' c = case go (ord c) of
                  (x, xs) -> (fromIntegral x, map fromIntegral xs)
 where
  go oc
   | oc <= 0x7f       = ( oc
                        , [
                        ])

   | oc <= 0x7ff      = ( 0xc0 + (oc `Data.Bits.shiftR` 6)
                        , [0x80 + oc Data.Bits..&. 0x3f
                        ])

   | oc <= 0xffff     = ( 0xe0 + (oc `Data.Bits.shiftR` 12)
                        , [0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ])
   | otherwise        = ( 0xf0 + (oc `Data.Bits.shiftR` 18)
                        , [0x80 + ((oc `Data.Bits.shiftR` 12) Data.Bits..&. 0x3f)
                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ])

#endif

type Byte = Word8

-- -----------------------------------------------------------------------------
-- The input type

#if defined(ALEX_POSN) || defined(ALEX_MONAD) || defined(ALEX_GSCAN)
type AlexInput = (AlexPosn,     -- current position,
                  Char,         -- previous char
                  [Byte],       -- pending bytes on current char
                  String)       -- current input string

ignorePendingBytes :: AlexInput -> AlexInput
ignorePendingBytes (p,c,_ps,s) = (p,c,[],s)

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar (_p,c,_bs,_s) = c

alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
alexGetByte (p,c,(b:bs),s) = Just (b,(p,c,bs,s))
alexGetByte (_,_,[],[]) = Nothing
alexGetByte (p,_,[],(c:s))  = let p' = alexMove p c
                              in case utf8Encode' c of
                                   (b, bs) -> p' `seq`  Just (b, (p', c, bs, s))
#endif

#if defined(ALEX_POSN_BYTESTRING) || defined(ALEX_MONAD_BYTESTRING)
type AlexInput = (AlexPosn,     -- current position,
                  Char,         -- previous char
                  ByteString.ByteString,        -- current input string
                  Int64)           -- bytes consumed so far

ignorePendingBytes :: AlexInput -> AlexInput
ignorePendingBytes i = i   -- no pending bytes when lexing bytestrings

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar (_,c,_,_) = c

alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
alexGetByte (p,_,cs,n) =
    case ByteString.uncons cs of
        Nothing -> Nothing
        Just (b, cs') ->
            let c   = ByteString.w2c b
                p'  = alexMove p c
                n'  = n+1
            in p' `seq` cs' `seq` n' `seq` Just (b, (p', c, cs',n'))
#endif

#ifdef ALEX_BASIC_BYTESTRING
data AlexInput = AlexInput { alexChar :: {-# UNPACK #-} !Char,      -- previous char
                             alexStr ::  !ByteString.ByteString,    -- current input string
                             alexBytePos :: {-# UNPACK #-} !Int64}  -- bytes consumed so far

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar = alexChar

alexGetByte (AlexInput {alexStr=cs,alexBytePos=n}) =
    case ByteString.uncons cs of
        Nothing -> Nothing
        Just (c, rest) ->
            Just (c, AlexInput {
                alexChar = ByteString.w2c c,
                alexStr =  rest,
                alexBytePos = n+1})
#endif

#ifdef ALEX_STRICT_BYTESTRING
data AlexInput = AlexInput { alexChar :: {-# UNPACK #-} !Char,
                             alexStr :: {-# UNPACK #-} !ByteString.ByteString,
                             alexBytePos :: {-# UNPACK #-} !Int}

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar = alexChar

alexGetByte (AlexInput {alexStr=cs,alexBytePos=n}) =
    case ByteString.uncons cs of
        Nothing -> Nothing
        Just (c, rest) ->
            Just (c, AlexInput {
                alexChar = ByteString.w2c c,
                alexStr =  rest,
                alexBytePos = n+1})
#endif

-- -----------------------------------------------------------------------------
-- Token positions

-- `Posn' records the location of a token in the input text.  It has three
-- fields: the address (number of chacaters preceding the token), line number
-- and column of a token within the file. `start_pos' gives the position of the
-- start of the file and `eof_pos' a standard encoding for the end of file.
-- `move_pos' calculates the new position after traversing a given character,
-- assuming the usual eight character tab stops.

#if defined(ALEX_POSN) || defined(ALEX_MONAD) || defined(ALEX_POSN_BYTESTRING) || defined(ALEX_MONAD_BYTESTRING) || defined(ALEX_GSCAN)
data AlexPosn = AlexPn !Int !Int !Int
        deriving (Eq,Show)

alexStartPos :: AlexPosn
alexStartPos = AlexPn 0 1 1

alexMove :: AlexPosn -> Char -> AlexPosn
alexMove (AlexPn a l c) '\t' = AlexPn (a+1)  l     (c+alex_tab_size-((c-1) `mod` alex_tab_size))
alexMove (AlexPn a l _) '\n' = AlexPn (a+1) (l+1)   1
alexMove (AlexPn a l c) _    = AlexPn (a+1)  l     (c+1)
#endif

-- -----------------------------------------------------------------------------
-- Monad (default and with ByteString input)

#if defined(ALEX_MONAD) || defined(ALEX_MONAD_BYTESTRING)
data AlexState = AlexState {
        alex_pos :: !AlexPosn,  -- position at current input location
#ifndef ALEX_MONAD_BYTESTRING
        alex_inp :: String,     -- the current input
        alex_chr :: !Char,      -- the character before the input
        alex_bytes :: [Byte],
#else /* ALEX_MONAD_BYTESTRING */
        alex_bpos:: !Int64,     -- bytes consumed so far
        alex_inp :: ByteString.ByteString,      -- the current input
        alex_chr :: !Char,      -- the character before the input
#endif /* ALEX_MONAD_BYTESTRING */
        alex_scd :: !Int        -- the current startcode
#ifdef ALEX_MONAD_USER_STATE
      , alex_ust :: AlexUserState -- AlexUserState will be defined in the user program
#endif
    }

-- Compile with -funbox-strict-fields for best results!

#ifndef ALEX_MONAD_BYTESTRING
runAlex :: String -> Alex a -> Either String a
runAlex input__ (Alex f)
   = case f (AlexState {alex_bytes = [],
#else /* ALEX_MONAD_BYTESTRING */
runAlex :: ByteString.ByteString -> Alex a -> Either String a
runAlex input__ (Alex f)
   = case f (AlexState {alex_bpos = 0,
#endif /* ALEX_MONAD_BYTESTRING */
                        alex_pos = alexStartPos,
                        alex_inp = input__,
                        alex_chr = '\n',
#ifdef ALEX_MONAD_USER_STATE
                        alex_ust = alexInitUserState,
#endif
                        alex_scd = 0}) of Left msg -> Left msg
                                          Right ( _, a ) -> Right a

newtype Alex a = Alex { unAlex :: AlexState -> Either String (AlexState, a) }

instance Functor Alex where
  fmap f a = Alex $ \s -> case unAlex a s of
                            Left msg -> Left msg
                            Right (s', a') -> Right (s', f a')

instance Applicative Alex where
  pure a   = Alex $ \s -> Right (s, a)
  fa <*> a = Alex $ \s -> case unAlex fa s of
                            Left msg -> Left msg
                            Right (s', f) -> case unAlex a s' of
                                               Left msg -> Left msg
                                               Right (s'', b) -> Right (s'', f b)

instance Monad Alex where
  m >>= k  = Alex $ \s -> case unAlex m s of
                                Left msg -> Left msg
                                Right (s',a) -> unAlex (k a) s'
  return = App.pure

alexGetInput :: Alex AlexInput
alexGetInput
#ifndef ALEX_MONAD_BYTESTRING
 = Alex $ \s@AlexState{alex_pos=pos,alex_chr=c,alex_bytes=bs,alex_inp=inp__} ->
        Right (s, (pos,c,bs,inp__))
#else /* ALEX_MONAD_BYTESTRING */
 = Alex $ \s@AlexState{alex_pos=pos,alex_bpos=bpos,alex_chr=c,alex_inp=inp__} ->
        Right (s, (pos,c,inp__,bpos))
#endif /* ALEX_MONAD_BYTESTRING */

alexSetInput :: AlexInput -> Alex ()
#ifndef ALEX_MONAD_BYTESTRING
alexSetInput (pos,c,bs,inp__)
 = Alex $ \s -> case s{alex_pos=pos,alex_chr=c,alex_bytes=bs,alex_inp=inp__} of
#else /* ALEX_MONAD_BYTESTRING */
alexSetInput (pos,c,inp__,bpos)
 = Alex $ \s -> case s{alex_pos=pos,
                       alex_bpos=bpos,
                       alex_chr=c,
                       alex_inp=inp__} of
#endif /* ALEX_MONAD_BYTESTRING */
                  state__@(AlexState{}) -> Right (state__, ())

alexError :: String -> Alex a
alexError message = Alex $ const $ Left message

alexGetStartCode :: Alex Int
alexGetStartCode = Alex $ \s@AlexState{alex_scd=sc} -> Right (s, sc)

alexSetStartCode :: Int -> Alex ()
alexSetStartCode sc = Alex $ \s -> Right (s{alex_scd=sc}, ())

#if !defined(ALEX_MONAD_BYTESTRING) && defined(ALEX_MONAD_USER_STATE)
alexGetUserState :: Alex AlexUserState
alexGetUserState = Alex $ \s@AlexState{alex_ust=ust} -> Right (s,ust)

alexSetUserState :: AlexUserState -> Alex ()
alexSetUserState ss = Alex $ \s -> Right (s{alex_ust=ss}, ())
#endif /* !defined(ALEX_MONAD_BYTESTRING) && defined(ALEX_MONAD_USER_STATE) */

alexMonadScan = do
#ifndef ALEX_MONAD_BYTESTRING
  inp__ <- alexGetInput
#else /* ALEX_MONAD_BYTESTRING */
  inp__@(_,_,_,n) <- alexGetInput
#endif /* ALEX_MONAD_BYTESTRING */
  sc <- alexGetStartCode
  case alexScan inp__ sc of
    AlexEOF -> alexEOF
    AlexError ((AlexPn _ line column),_,_,_) -> alexError $ "lexical error at line " ++ (show line) ++ ", column " ++ (show column)
    AlexSkip  inp__' _len -> do
        alexSetInput inp__'
        alexMonadScan
#ifndef ALEX_MONAD_BYTESTRING
    AlexToken inp__' len action -> do
#else /* ALEX_MONAD_BYTESTRING */
    AlexToken inp__'@(_,_,_,n') _ action -> let len = n'-n in do
#endif /* ALEX_MONAD_BYTESTRING */
        alexSetInput inp__'
        action (ignorePendingBytes inp__) len

-- -----------------------------------------------------------------------------
-- Useful token actions

#ifndef ALEX_MONAD_BYTESTRING
type AlexAction result = AlexInput -> Int -> Alex result
#else /* ALEX_MONAD_BYTESTRING */
type AlexAction result = AlexInput -> Int64 -> Alex result
#endif /* ALEX_MONAD_BYTESTRING */

-- just ignore this token and scan another one
-- skip :: AlexAction result
skip _input _len = alexMonadScan

-- ignore this token, but set the start code to a new value
-- begin :: Int -> AlexAction result
begin code _input _len = do alexSetStartCode code; alexMonadScan

-- perform an action for this token, and set the start code to a new value
andBegin :: AlexAction result -> Int -> AlexAction result
(action `andBegin` code) input__ len = do
  alexSetStartCode code
  action input__ len

#ifndef ALEX_MONAD_BYTESTRING
token :: (AlexInput -> Int -> token) -> AlexAction token
#else /* ALEX_MONAD_BYTESTRING */
token :: (AlexInput -> Int64 -> token) -> AlexAction token
#endif /* ALEX_MONAD_BYTESTRING */
token t input__ len = return (t input__ len)
#endif /* defined(ALEX_MONAD) || defined(ALEX_MONAD_BYTESTRING) */


-- -----------------------------------------------------------------------------
-- Basic wrapper

#ifdef ALEX_BASIC
type AlexInput = (Char,[Byte],String)

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar (c,_,_) = c

-- alexScanTokens :: String -> [token]
alexScanTokens str = go ('\n',[],str)
  where go inp__@(_,_bs,s) =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError _ -> error "lexical error"
                AlexSkip  inp__' _ln     -> go inp__'
                AlexToken inp__' len act -> act (take len s) : go inp__'

alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
alexGetByte (c,(b:bs),s) = Just (b,(c,bs,s))
alexGetByte (_,[],[])    = Nothing
alexGetByte (_,[],(c:s)) = case utf8Encode' c of
                             (b, bs) -> Just (b, (c, bs, s))
#endif


-- -----------------------------------------------------------------------------
-- Basic wrapper, ByteString version

#ifdef ALEX_BASIC_BYTESTRING

-- alexScanTokens :: ByteString.ByteString -> [token]
alexScanTokens str = go (AlexInput '\n' str 0)
  where go inp__ =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError _ -> error "lexical error"
                AlexSkip  inp__' _len  -> go inp__'
                AlexToken inp__' _ act ->
                  let len = alexBytePos inp__' - alexBytePos inp__ in
                  act (ByteString.take len (alexStr inp__)) : go inp__'

#endif

#ifdef ALEX_STRICT_BYTESTRING

-- alexScanTokens :: ByteString.ByteString -> [token]
alexScanTokens str = go (AlexInput '\n' str 0)
  where go inp__ =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError _ -> error "lexical error"
                AlexSkip  inp__' _len  -> go inp__'
                AlexToken inp__' _ act ->
                  let len = alexBytePos inp__' - alexBytePos inp__ in
                  act (ByteString.take len (alexStr inp__)) : go inp__'

#endif


-- -----------------------------------------------------------------------------
-- Posn wrapper

-- Adds text positions to the basic model.

#ifdef ALEX_POSN
--alexScanTokens :: String -> [token]
alexScanTokens str0 = go (alexStartPos,'\n',[],str0)
  where go inp__@(pos,_,_,str) =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError ((AlexPn _ line column),_,_,_) -> error $ "lexical error at line " ++ (show line) ++ ", column " ++ (show column)
                AlexSkip  inp__' _ln     -> go inp__'
                AlexToken inp__' len act -> act pos (take len str) : go inp__'
#endif


-- -----------------------------------------------------------------------------
-- Posn wrapper, ByteString version

#ifdef ALEX_POSN_BYTESTRING
--alexScanTokens :: ByteString.ByteString -> [token]
alexScanTokens str0 = go (alexStartPos,'\n',str0,0)
  where go inp__@(pos,_,str,n) =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError ((AlexPn _ line column),_,_,_) -> error $ "lexical error at line " ++ (show line) ++ ", column " ++ (show column)
                AlexSkip  inp__' _len       -> go inp__'
                AlexToken inp__'@(_,_,_,n') _ act ->
                  act pos (ByteString.take (n'-n) str) : go inp__'
#endif


-- -----------------------------------------------------------------------------
-- GScan wrapper

-- For compatibility with previous versions of Alex, and because we can.

#ifdef ALEX_GSCAN
alexGScan stop__ state__ inp__ =
  alex_gscan stop__ alexStartPos '\n' [] inp__ (0,state__)

alex_gscan stop__ p c bs inp__ (sc,state__) =
  case alexScan (p,c,bs,inp__) sc of
        AlexEOF     -> stop__ p c inp__ (sc,state__)
        AlexError _ -> stop__ p c inp__ (sc,state__)
        AlexSkip (p',c',bs',inp__') _len ->
          alex_gscan stop__ p' c' bs' inp__' (sc,state__)
        AlexToken (p',c',bs',inp__') len k ->
           k p c inp__ len (\scs -> alex_gscan stop__ p' c' bs' inp__' scs)                  (sc,state__)
#endif
alex_tab_size :: Int
alex_tab_size = 8
alex_base :: Array Int Int
alex_base = listArray (0 :: Int, 229)
  [ -8
  , 79
  , 166
  , 253
  , 340
  , 427
  , 514
  , 601
  , 688
  , 775
  , 862
  , 949
  , 1036
  , 1123
  , 1210
  , 1297
  , 1384
  , 1471
  , 1558
  , 1645
  , 1732
  , 1819
  , 1906
  , 1993
  , 2080
  , 2167
  , 2254
  , 2341
  , 2428
  , 2515
  , 2602
  , 2689
  , 2776
  , 2863
  , 2950
  , 3037
  , 3124
  , 3211
  , 3298
  , 3385
  , 3472
  , 3559
  , 3646
  , 3733
  , -42
  , 3820
  , 3907
  , 3994
  , 4081
  , 4168
  , 4255
  , 4342
  , 4429
  , 4516
  , 4603
  , -104
  , 0
  , 0
  , -103
  , 0
  , 0
  , 0
  , -87
  , 0
  , -92
  , 0
  , -82
  , 22
  , -80
  , 0
  , 4690
  , 4777
  , 4864
  , 4951
  , 5038
  , 5125
  , 5212
  , 5299
  , 5386
  , 5473
  , 5560
  , 5647
  , 5734
  , 5821
  , 5908
  , 5995
  , 6082
  , 6169
  , 6256
  , 6343
  , 6430
  , 6517
  , 6604
  , 6691
  , 6778
  , 6865
  , 6952
  , 7039
  , 7126
  , 7213
  , 7300
  , 7387
  , 7474
  , 7561
  , 7648
  , 7735
  , 7822
  , 7909
  , 0
  , 197
  , 0
  , 7996
  , 8083
  , 8170
  , 8257
  , 8344
  , 8431
  , 8518
  , 8605
  , 8692
  , 8779
  , 8866
  , 8953
  , 9040
  , 9127
  , 9214
  , 9301
  , 9388
  , 9475
  , 9562
  , 9649
  , 9736
  , 9823
  , 9910
  , 9997
  , 10084
  , 10171
  , 10258
  , 10345
  , 10432
  , 10519
  , 10606
  , 10693
  , 10780
  , 10867
  , 10954
  , 11041
  , 11128
  , -94
  , -81
  , -91
  , 19
  , 11123
  , 11379
  , 11380
  , 11508
  , 11572
  , 11637
  , 11750
  , 0
  , 0
  , 0
  , 11996
  , 11815
  , 0
  , 12124
  , 0
  , 12237
  , 12302
  , 12558
  , 12559
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 12779
  , 12866
  , 12953
  , 13040
  , 13127
  , 13214
  , 13301
  , 13388
  , 13475
  , 13562
  , 13649
  , 13736
  , 13823
  , 13910
  , 13997
  , 14084
  , 14171
  , 14258
  , 14345
  , 14432
  , 14519
  , 14606
  , 14693
  , 14780
  , 14867
  , 14954
  , 15041
  , 15128
  , 15215
  , 15302
  , 15389
  , 15476
  , 15563
  , 15650
  , 15737
  , 15824
  , 15911
  , 15998
  , 16085
  , 16172
  , 16259
  , 16346
  , 16433
  , 16520
  , 16607
  , 16694
  , 16781
  , 16868
  , 16955
  , 17042
  , 17129
  , 17216
  ]

alex_table :: Array Int Int
alex_table = listArray (0 :: Int, 17471)
  [ 0
  , 109
  , 109
  , 109
  , 109
  , 109
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 56
  , 59
  , 63
  , 67
  , 65
  , 110
  , 148
  , 149
  , 109
  , 61
  , 0
  , 151
  , 57
  , 66
  , 0
  , 0
  , 69
  , 60
  , 177
  , 175
  , 174
  , 176
  , 162
  , 0
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 44
  , 173
  , 152
  , 0
  , 0
  , 0
  , 0
  , 0
  , 146
  , 107
  , 136
  , 71
  , 132
  , 107
  , 107
  , 107
  , 126
  , 75
  , 107
  , 72
  , 92
  , 134
  , 191
  , 128
  , 107
  , 76
  , 138
  , 107
  , 107
  , 107
  , 107
  , 89
  , 107
  , 107
  , 171
  , 0
  , 172
  , 0
  , 0
  , 0
  , 81
  , 107
  , 93
  , 121
  , 113
  , 107
  , 107
  , 107
  , 103
  , 118
  , 107
  , 123
  , 112
  , 117
  , 100
  , 99
  , 107
  , 101
  , 90
  , 45
  , 107
  , 107
  , 107
  , 116
  , 107
  , 107
  , 107
  , 150
  , 0
  , 107
  , 55
  , 64
  , 58
  , 62
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 68
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 109
  , 109
  , 109
  , 109
  , 109
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 109
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 4
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 9
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 14
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 16
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 31
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 19
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 23
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 25
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 27
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 30
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 32
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 36
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 38
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 40
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 42
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 45
  , 45
  , 45
  , 45
  , 45
  , 45
  , 45
  , 45
  , 45
  , 45
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 49
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 207
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 73
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 145
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 70
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 41
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 198
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 219
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 209
  , 107
  , 214
  , 107
  , 107
  , 107
  , 107
  , 212
  , 206
  , 211
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 77
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 43
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 217
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 203
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 194
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 224
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 223
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 84
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 83
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 35
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 1
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 86
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 11
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 26
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 12
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 193
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 91
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 226
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 80
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 228
  , 107
  , 107
  , 107
  , 107
  , 107
  , 229
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 179
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 87
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 18
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 10
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 53
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 47
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 33
  , 107
  , 107
  , 107
  , 6
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 98
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 29
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 94
  , 107
  , 107
  , 107
  , 107
  , 39
  , 85
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 2
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 102
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 28
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 104
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 129
  , 120
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 13
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 221
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 46
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 52
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 50
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 105
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 106
  , 95
  , 48
  , 111
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 97
  , 107
  , 107
  , 107
  , 107
  , 114
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 51
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 54
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 5
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 3
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 115
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 96
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 20
  , 107
  , 24
  , 107
  , 107
  , 107
  , 107
  , 22
  , 15
  , 119
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 21
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 7
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 8
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 82
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 17
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 122
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 204
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 124
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 125
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 78
  , 197
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 218
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 200
  , 107
  , 107
  , 107
  , 107
  , 37
  , 142
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 88
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 127
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 130
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 131
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 220
  , 107
  , 107
  , 107
  , 195
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 79
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 133
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 208
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 140
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 135
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 186
  , 107
  , 107
  , 107
  , 107
  , 107
  , 187
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 137
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 184
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 147
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 216
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 139
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 201
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 141
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 34
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 143
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 144
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 181
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 189
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , -1
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 182
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 153
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 156
  , 154
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 157
  , 155
  , 161
  , 161
  , 161
  , 158
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 0
  , 153
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 159
  , 154
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , 160
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 156
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 157
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 169
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 163
  , 170
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 168
  , 165
  , 166
  , 166
  , 166
  , 167
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 170
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 164
  , 168
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 163
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 0
  , 169
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 108
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 178
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 180
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 183
  , 107
  , 107
  , 107
  , 107
  , 107
  , 185
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 188
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 190
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 192
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 196
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 199
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 202
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 205
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 74
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 210
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 213
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 215
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 222
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 225
  , 107
  , 107
  , 107
  , 107
  , 107
  , 227
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 107
  , 0
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 107
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  ]

alex_check :: Array Int Int
alex_check = listArray (0 :: Int, 17471)
  [ -1
  , 9
  , 10
  , 11
  , 12
  , 13
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 120
  , 120
  , 105
  , 101
  , 112
  , 115
  , 103
  , 114
  , 32
  , 105
  , -1
  , 35
  , 120
  , 37
  , -1
  , -1
  , 112
  , 120
  , 42
  , 43
  , 44
  , 45
  , 46
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 32
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , -1
  , 93
  , -1
  , -1
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , 97
  , -1
  , 39
  , 97
  , 98
  , 99
  , 100
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 115
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , 9
  , 10
  , 11
  , 12
  , 13
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , 32
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , 10
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 0
  , 1
  , 2
  , 3
  , 4
  , 5
  , 6
  , 7
  , 8
  , 9
  , 10
  , 11
  , 12
  , 13
  , 14
  , 15
  , 16
  , 17
  , 18
  , 19
  , 20
  , 21
  , 22
  , 23
  , 24
  , 25
  , 26
  , 27
  , 28
  , 29
  , 30
  , 31
  , 32
  , 33
  , 34
  , 35
  , 36
  , 37
  , 38
  , 39
  , 40
  , 41
  , 42
  , 43
  , 44
  , 45
  , 46
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 59
  , 60
  , 61
  , 62
  , 63
  , 64
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , 92
  , 93
  , 94
  , 95
  , 96
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 123
  , 124
  , 125
  , 126
  , 127
  , -1
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 10
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 0
  , 1
  , 2
  , 3
  , 4
  , 5
  , 6
  , 7
  , 8
  , 9
  , 10
  , 11
  , 12
  , 13
  , 14
  , 15
  , 16
  , 17
  , 18
  , 19
  , 20
  , 21
  , 22
  , 23
  , 24
  , 25
  , 26
  , 27
  , 28
  , 29
  , 30
  , 31
  , 32
  , 33
  , 34
  , 35
  , 36
  , 37
  , 38
  , 39
  , 40
  , 41
  , 42
  , 43
  , 44
  , 45
  , 46
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 59
  , 60
  , 61
  , 62
  , 63
  , 64
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , 92
  , 93
  , 94
  , 95
  , 96
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 123
  , 124
  , 125
  , 126
  , 127
  , -1
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 36
  , -1
  , -1
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_deflt :: Array Int Int
alex_deflt = listArray (0 :: Int, 229)
  [ -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 162
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 152
  , 152
  , -1
  , -1
  , 152
  , 159
  , 160
  , 152
  , 159
  , 160
  , 162
  , 162
  , 108
  , -1
  , 164
  , 164
  , 108
  , 162
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_accept = listArray (0 :: Int, 229)
  [ AlexAccNone
  , AlexAcc 196
  , AlexAcc 195
  , AlexAcc 194
  , AlexAcc 193
  , AlexAcc 192
  , AlexAcc 191
  , AlexAcc 190
  , AlexAcc 189
  , AlexAcc 188
  , AlexAcc 187
  , AlexAcc 186
  , AlexAcc 185
  , AlexAcc 184
  , AlexAcc 183
  , AlexAcc 182
  , AlexAcc 181
  , AlexAcc 180
  , AlexAcc 179
  , AlexAcc 178
  , AlexAcc 177
  , AlexAcc 176
  , AlexAcc 175
  , AlexAcc 174
  , AlexAcc 173
  , AlexAcc 172
  , AlexAcc 171
  , AlexAcc 170
  , AlexAcc 169
  , AlexAcc 168
  , AlexAcc 167
  , AlexAcc 166
  , AlexAcc 165
  , AlexAcc 164
  , AlexAcc 163
  , AlexAcc 162
  , AlexAcc 161
  , AlexAcc 160
  , AlexAcc 159
  , AlexAcc 158
  , AlexAcc 157
  , AlexAcc 156
  , AlexAcc 155
  , AlexAcc 154
  , AlexAcc 153
  , AlexAcc 152
  , AlexAcc 151
  , AlexAcc 150
  , AlexAcc 149
  , AlexAcc 148
  , AlexAcc 147
  , AlexAcc 146
  , AlexAcc 145
  , AlexAcc 144
  , AlexAcc 143
  , AlexAccNone
  , AlexAcc 142
  , AlexAcc 141
  , AlexAccNone
  , AlexAcc 140
  , AlexAcc 139
  , AlexAcc 138
  , AlexAccNone
  , AlexAcc 137
  , AlexAccNone
  , AlexAcc 136
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAcc 135
  , AlexAcc 134
  , AlexAcc 133
  , AlexAcc 132
  , AlexAcc 131
  , AlexAcc 130
  , AlexAcc 129
  , AlexAcc 128
  , AlexAcc 127
  , AlexAcc 126
  , AlexAcc 125
  , AlexAcc 124
  , AlexAcc 123
  , AlexAcc 122
  , AlexAcc 121
  , AlexAcc 120
  , AlexAcc 119
  , AlexAcc 118
  , AlexAcc 117
  , AlexAcc 116
  , AlexAcc 115
  , AlexAcc 114
  , AlexAcc 113
  , AlexAcc 112
  , AlexAcc 111
  , AlexAcc 110
  , AlexAcc 109
  , AlexAcc 108
  , AlexAcc 107
  , AlexAcc 106
  , AlexAcc 105
  , AlexAcc 104
  , AlexAcc 103
  , AlexAcc 102
  , AlexAcc 101
  , AlexAcc 100
  , AlexAcc 99
  , AlexAcc 98
  , AlexAcc 97
  , AlexAccNone
  , AlexAccSkip
  , AlexAcc 96
  , AlexAcc 95
  , AlexAcc 94
  , AlexAcc 93
  , AlexAcc 92
  , AlexAcc 91
  , AlexAcc 90
  , AlexAcc 89
  , AlexAcc 88
  , AlexAcc 87
  , AlexAcc 86
  , AlexAcc 85
  , AlexAcc 84
  , AlexAcc 83
  , AlexAcc 82
  , AlexAcc 81
  , AlexAcc 80
  , AlexAcc 79
  , AlexAcc 78
  , AlexAcc 77
  , AlexAcc 76
  , AlexAcc 75
  , AlexAcc 74
  , AlexAcc 73
  , AlexAcc 72
  , AlexAcc 71
  , AlexAcc 70
  , AlexAcc 69
  , AlexAcc 68
  , AlexAcc 67
  , AlexAcc 66
  , AlexAcc 65
  , AlexAcc 64
  , AlexAcc 63
  , AlexAcc 62
  , AlexAcc 61
  , AlexAcc 60
  , AlexAcc 59
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccSkip
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccSkip
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAcc 58
  , AlexAcc 57
  , AlexAcc 56
  , AlexAcc 55
  , AlexAcc 54
  , AlexAcc 53
  , AlexAcc 52
  , AlexAcc 51
  , AlexAcc 50
  , AlexAcc 49
  , AlexAcc 48
  , AlexAcc 47
  , AlexAcc 46
  , AlexAcc 45
  , AlexAcc 44
  , AlexAcc 43
  , AlexAcc 42
  , AlexAcc 41
  , AlexAcc 40
  , AlexAcc 39
  , AlexAcc 38
  , AlexAcc 37
  , AlexAcc 36
  , AlexAcc 35
  , AlexAcc 34
  , AlexAcc 33
  , AlexAcc 32
  , AlexAcc 31
  , AlexAcc 30
  , AlexAcc 29
  , AlexAcc 28
  , AlexAcc 27
  , AlexAcc 26
  , AlexAcc 25
  , AlexAcc 24
  , AlexAcc 23
  , AlexAcc 22
  , AlexAcc 21
  , AlexAcc 20
  , AlexAcc 19
  , AlexAcc 18
  , AlexAcc 17
  , AlexAcc 16
  , AlexAcc 15
  , AlexAcc 14
  , AlexAcc 13
  , AlexAcc 12
  , AlexAcc 11
  , AlexAcc 10
  , AlexAcc 9
  , AlexAcc 8
  , AlexAcc 7
  , AlexAcc 6
  , AlexAcc 5
  , AlexAcc 4
  , AlexAcc 3
  , AlexAcc 2
  , AlexAcc 1
  , AlexAcc 0
  ]

alex_actions = array (0 :: Int, 197)
  [ (196,alex_action_51)
  , (195,alex_action_52)
  , (194,alex_action_100)
  , (193,alex_action_53)
  , (192,alex_action_54)
  , (191,alex_action_55)
  , (190,alex_action_56)
  , (189,alex_action_100)
  , (188,alex_action_57)
  , (187,alex_action_58)
  , (186,alex_action_59)
  , (185,alex_action_60)
  , (184,alex_action_100)
  , (183,alex_action_61)
  , (182,alex_action_100)
  , (181,alex_action_62)
  , (180,alex_action_63)
  , (179,alex_action_100)
  , (178,alex_action_64)
  , (177,alex_action_65)
  , (176,alex_action_66)
  , (175,alex_action_67)
  , (174,alex_action_68)
  , (173,alex_action_69)
  , (172,alex_action_70)
  , (171,alex_action_100)
  , (170,alex_action_71)
  , (169,alex_action_72)
  , (168,alex_action_100)
  , (167,alex_action_73)
  , (166,alex_action_100)
  , (165,alex_action_74)
  , (164,alex_action_75)
  , (163,alex_action_76)
  , (162,alex_action_100)
  , (161,alex_action_77)
  , (160,alex_action_100)
  , (159,alex_action_78)
  , (158,alex_action_100)
  , (157,alex_action_79)
  , (156,alex_action_100)
  , (155,alex_action_80)
  , (154,alex_action_81)
  , (153,alex_action_82)
  , (152,alex_action_83)
  , (151,alex_action_84)
  , (150,alex_action_85)
  , (149,alex_action_100)
  , (148,alex_action_86)
  , (147,alex_action_87)
  , (146,alex_action_88)
  , (145,alex_action_89)
  , (144,alex_action_90)
  , (143,alex_action_91)
  , (142,alex_action_92)
  , (141,alex_action_93)
  , (140,alex_action_94)
  , (139,alex_action_95)
  , (138,alex_action_96)
  , (137,alex_action_97)
  , (136,alex_action_98)
  , (135,alex_action_99)
  , (134,alex_action_100)
  , (133,alex_action_100)
  , (132,alex_action_100)
  , (131,alex_action_100)
  , (130,alex_action_100)
  , (129,alex_action_100)
  , (128,alex_action_100)
  , (127,alex_action_100)
  , (126,alex_action_100)
  , (125,alex_action_100)
  , (124,alex_action_100)
  , (123,alex_action_100)
  , (122,alex_action_100)
  , (121,alex_action_100)
  , (120,alex_action_100)
  , (119,alex_action_100)
  , (118,alex_action_100)
  , (117,alex_action_100)
  , (116,alex_action_100)
  , (115,alex_action_100)
  , (114,alex_action_100)
  , (113,alex_action_100)
  , (112,alex_action_100)
  , (111,alex_action_100)
  , (110,alex_action_100)
  , (109,alex_action_100)
  , (108,alex_action_100)
  , (107,alex_action_100)
  , (106,alex_action_100)
  , (105,alex_action_100)
  , (104,alex_action_100)
  , (103,alex_action_100)
  , (102,alex_action_100)
  , (101,alex_action_100)
  , (100,alex_action_100)
  , (99,alex_action_100)
  , (98,alex_action_100)
  , (97,alex_action_100)
  , (96,alex_action_1)
  , (95,alex_action_100)
  , (94,alex_action_100)
  , (93,alex_action_100)
  , (92,alex_action_100)
  , (91,alex_action_100)
  , (90,alex_action_100)
  , (89,alex_action_100)
  , (88,alex_action_100)
  , (87,alex_action_100)
  , (86,alex_action_100)
  , (85,alex_action_100)
  , (84,alex_action_100)
  , (83,alex_action_100)
  , (82,alex_action_100)
  , (81,alex_action_100)
  , (80,alex_action_100)
  , (79,alex_action_100)
  , (78,alex_action_100)
  , (77,alex_action_100)
  , (76,alex_action_100)
  , (75,alex_action_100)
  , (74,alex_action_100)
  , (73,alex_action_100)
  , (72,alex_action_100)
  , (71,alex_action_100)
  , (70,alex_action_100)
  , (69,alex_action_100)
  , (68,alex_action_100)
  , (67,alex_action_100)
  , (66,alex_action_100)
  , (65,alex_action_100)
  , (64,alex_action_100)
  , (63,alex_action_100)
  , (62,alex_action_100)
  , (61,alex_action_100)
  , (60,alex_action_100)
  , (59,alex_action_100)
  , (58,alex_action_4)
  , (57,alex_action_5)
  , (56,alex_action_6)
  , (55,alex_action_7)
  , (54,alex_action_9)
  , (53,alex_action_10)
  , (52,alex_action_11)
  , (51,alex_action_12)
  , (50,alex_action_100)
  , (49,alex_action_13)
  , (48,alex_action_100)
  , (47,alex_action_14)
  , (46,alex_action_15)
  , (45,alex_action_100)
  , (44,alex_action_16)
  , (43,alex_action_17)
  , (42,alex_action_18)
  , (41,alex_action_19)
  , (40,alex_action_100)
  , (39,alex_action_20)
  , (38,alex_action_100)
  , (37,alex_action_21)
  , (36,alex_action_100)
  , (35,alex_action_22)
  , (34,alex_action_23)
  , (33,alex_action_24)
  , (32,alex_action_100)
  , (31,alex_action_25)
  , (30,alex_action_26)
  , (29,alex_action_100)
  , (28,alex_action_27)
  , (27,alex_action_28)
  , (26,alex_action_100)
  , (25,alex_action_29)
  , (24,alex_action_30)
  , (23,alex_action_100)
  , (22,alex_action_31)
  , (21,alex_action_32)
  , (20,alex_action_33)
  , (19,alex_action_34)
  , (18,alex_action_100)
  , (17,alex_action_35)
  , (16,alex_action_36)
  , (15,alex_action_37)
  , (14,alex_action_38)
  , (13,alex_action_39)
  , (12,alex_action_40)
  , (11,alex_action_41)
  , (10,alex_action_42)
  , (9,alex_action_43)
  , (8,alex_action_44)
  , (7,alex_action_45)
  , (6,alex_action_100)
  , (5,alex_action_46)
  , (4,alex_action_47)
  , (3,alex_action_100)
  , (2,alex_action_48)
  , (1,alex_action_49)
  , (0,alex_action_50)
  ]

{-# LINE 126 "LexIntel.x" #-}
-- Each action has type :: String -> Token

-- The token type is in Token.hs
{-
main = do
  s <- getContents
  print (alexScanTokens s)
-}
alex_action_1 = \p s -> Args p
alex_action_4 = \ p s -> LBrack p
alex_action_5 = \ p s -> RBrack p
alex_action_6 = \ p s -> Colon p
alex_action_7 = \ p s -> Comma p
alex_action_8 = \ p s -> Dot p
alex_action_9 = \ p s -> Plus p
alex_action_10 = \ p s -> Minus p
alex_action_11 = \ p s -> Times p
alex_action_12 = \ p s -> Mov p
alex_action_13 = \ p s -> Add p
alex_action_14 = \ p s -> Sub p
alex_action_15 = \ p s -> Shl p
alex_action_16 = \ p s -> Shr p
alex_action_17 = \ p s -> Sal p
alex_action_18 = \ p s -> Sar p
alex_action_19 = \ p s -> And p
alex_action_20 = \ p s -> Or p
alex_action_21 = \ p s -> Xor p
alex_action_22 = \ p s -> Neg p
alex_action_23 = \ p s -> Not p
alex_action_24 = \ p s -> Inc p
alex_action_25 = \ p s -> Dec p
alex_action_26 = \ p s -> Pop p
alex_action_27 = \ p s -> Push p
alex_action_28 = \ p s -> Imul p
alex_action_29 = \ p s -> Idiv p
alex_action_30 = \ p s -> Jmp p
alex_action_31 = \ p s -> Lea p
alex_action_32 = \ p s -> Cmp p
alex_action_33 = \ p s -> Je p
alex_action_34 = \ p s -> Jne p
alex_action_35 = \ p s -> Jl p
alex_action_36 = \ p s -> Jle p
alex_action_37 = \ p s -> Jg p
alex_action_38 = \ p s -> Jge p
alex_action_39 = \ p s -> Call p
alex_action_40 = \ p s -> Ret p
alex_action_41 = \ p s -> Enter p
alex_action_42 = \ p s -> Leave p
alex_action_43 = \ p s -> Nop p
alex_action_44 = \ p s -> Mov p
alex_action_45 = \ p s -> Add p
alex_action_46 = \ p s -> Sub p
alex_action_47 = \ p s -> Shl p
alex_action_48 = \ p s -> Shr p
alex_action_49 = \ p s -> Sal p
alex_action_50 = \ p s -> Sar p
alex_action_51 = \ p s -> And p
alex_action_52 = \ p s -> Or p
alex_action_53 = \ p s -> Xor p
alex_action_54 = \ p s -> Neg p
alex_action_55 = \ p s -> Not p
alex_action_56 = \ p s -> Inc p
alex_action_57 = \ p s -> Dec p
alex_action_58 = \ p s -> Pop p
alex_action_59 = \ p s -> Push p
alex_action_60 = \ p s -> Imul p
alex_action_61 = \ p s -> Idiv p
alex_action_62 = \ p s -> Jmp p
alex_action_63 = \ p s -> Lea p
alex_action_64 = \ p s -> Cmp p
alex_action_65 = \ p s -> Je p
alex_action_66 = \ p s -> Jne p
alex_action_67 = \ p s -> Jl p
alex_action_68 = \ p s -> Jle p
alex_action_69 = \ p s -> Jg p
alex_action_70 = \ p s -> Jge p
alex_action_71 = \ p s -> Call p
alex_action_72 = \ p s -> Ret p
alex_action_73 = \ p s -> Enter p
alex_action_74 = \ p s -> Leave p
alex_action_75 = \ p s -> Nop p
alex_action_76 = \ p s -> Dword p
alex_action_77 = \ p s -> Dword p
alex_action_78 = \ p s -> Token.Ptr p
alex_action_79 = \ p s -> Token.Ptr p
alex_action_80 = \ p s -> LOC p
alex_action_81 = \ p s -> REG p
alex_action_82 = \ p s -> Nat (read s) p
alex_action_83 = \ p ('t':s) -> Temp (read s) p
alex_action_84 = \ p s -> Reg eax p
alex_action_85 = \ p s -> Reg ebx p
alex_action_86 = \ p s -> Reg ecx p
alex_action_87 = \ p s -> Reg edx p
alex_action_88 = \ p s -> Reg esi p
alex_action_89 = \ p s -> Reg edi p
alex_action_90 = \ p s -> Reg ebp p
alex_action_91 = \ p s -> Reg esp p
alex_action_92 = \ p s -> Reg eax p
alex_action_93 = \ p s -> Reg ebx p
alex_action_94 = \ p s -> Reg ecx p
alex_action_95 = \ p s -> Reg edx p
alex_action_96 = \ p s -> Reg esi p
alex_action_97 = \ p s -> Reg edi p
alex_action_98 = \ p s -> Reg ebp p
alex_action_99 = \ p s -> Reg esp p
alex_action_100 = \ p s -> Id s p

#define ALEX_NOPRED 1
-- -----------------------------------------------------------------------------
-- ALEX TEMPLATE
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.

-- -----------------------------------------------------------------------------
-- INTERNALS and main scanner engine

#ifdef ALEX_GHC
#  define ILIT(n) n#
#  define IBOX(n) (I# (n))
#  define FAST_INT Int#
-- Do not remove this comment. Required to fix CPP parsing when using GCC and a clang-compiled alex.
#  if __GLASGOW_HASKELL__ > 706
#    define GTE(n,m) (tagToEnum# (n >=# m))
#    define EQ(n,m) (tagToEnum# (n ==# m))
#  else
#    define GTE(n,m) (n >=# m)
#    define EQ(n,m) (n ==# m)
#  endif
#  define PLUS(n,m) (n +# m)
#  define MINUS(n,m) (n -# m)
#  define TIMES(n,m) (n *# m)
#  define NEGATE(n) (negateInt# (n))
#  define IF_GHC(x) (x)
#else
#  define ILIT(n) (n)
#  define IBOX(n) (n)
#  define FAST_INT Int
#  define GTE(n,m) (n >= m)
#  define EQ(n,m) (n == m)
#  define PLUS(n,m) (n + m)
#  define MINUS(n,m) (n - m)
#  define TIMES(n,m) (n * m)
#  define NEGATE(n) (negate (n))
#  define IF_GHC(x)
#endif

#ifdef ALEX_GHC
data AlexAddr = AlexA# Addr#
-- Do not remove this comment. Required to fix CPP parsing when using GCC and a clang-compiled alex.
#if __GLASGOW_HASKELL__ < 503
uncheckedShiftL# = shiftL#
#endif

{-# INLINE alexIndexInt16OffAddr #-}
alexIndexInt16OffAddr :: AlexAddr -> Int# -> Int#
alexIndexInt16OffAddr (AlexA# arr) off =
#ifdef WORDS_BIGENDIAN
  narrow16Int# i
  where
        i    = word2Int# ((high `uncheckedShiftL#` 8#) `or#` low)
        high = int2Word# (ord# (indexCharOffAddr# arr (off' +# 1#)))
        low  = int2Word# (ord# (indexCharOffAddr# arr off'))
        off' = off *# 2#
#else
#if __GLASGOW_HASKELL__ >= 901
  int16ToInt#
#endif
    (indexInt16OffAddr# arr off)
#endif
#else
alexIndexInt16OffAddr arr off = arr ! off
#endif

#ifdef ALEX_GHC
{-# INLINE alexIndexInt32OffAddr #-}
alexIndexInt32OffAddr :: AlexAddr -> Int# -> Int#
alexIndexInt32OffAddr (AlexA# arr) off =
#ifdef WORDS_BIGENDIAN
  narrow32Int# i
  where
   i    = word2Int# ((b3 `uncheckedShiftL#` 24#) `or#`
                     (b2 `uncheckedShiftL#` 16#) `or#`
                     (b1 `uncheckedShiftL#` 8#) `or#` b0)
   b3   = int2Word# (ord# (indexCharOffAddr# arr (off' +# 3#)))
   b2   = int2Word# (ord# (indexCharOffAddr# arr (off' +# 2#)))
   b1   = int2Word# (ord# (indexCharOffAddr# arr (off' +# 1#)))
   b0   = int2Word# (ord# (indexCharOffAddr# arr off'))
   off' = off *# 4#
#else
#if __GLASGOW_HASKELL__ >= 901
  int32ToInt#
#endif
    (indexInt32OffAddr# arr off)
#endif
#else
alexIndexInt32OffAddr arr off = arr ! off
#endif

#ifdef ALEX_GHC

#if __GLASGOW_HASKELL__ < 503
quickIndex arr i = arr ! i
#else
-- GHC >= 503, unsafeAt is available from Data.Array.Base.
quickIndex = unsafeAt
#endif
#else
quickIndex arr i = arr ! i
#endif

-- -----------------------------------------------------------------------------
-- Main lexing routines

data AlexReturn a
  = AlexEOF
  | AlexError  !AlexInput
  | AlexSkip   !AlexInput !Int
  | AlexToken  !AlexInput !Int a

-- alexScan :: AlexInput -> StartCode -> AlexReturn a
alexScan input__ IBOX(sc)
  = alexScanUser undefined input__ IBOX(sc)

alexScanUser user__ input__ IBOX(sc)
  = case alex_scan_tkn user__ input__ ILIT(0) input__ sc AlexNone of
  (AlexNone, input__') ->
    case alexGetByte input__ of
      Nothing ->
#ifdef ALEX_DEBUG
                                   trace ("End of input.") $
#endif
                                   AlexEOF
      Just _ ->
#ifdef ALEX_DEBUG
                                   trace ("Error.") $
#endif
                                   AlexError input__'

  (AlexLastSkip input__'' len, _) ->
#ifdef ALEX_DEBUG
    trace ("Skipping.") $
#endif
    AlexSkip input__'' len

  (AlexLastAcc k input__''' len, _) ->
#ifdef ALEX_DEBUG
    trace ("Accept.") $
#endif
    AlexToken input__''' len (alex_actions ! k)


-- Push the input through the DFA, remembering the most recent accepting
-- state it encountered.

alex_scan_tkn user__ orig_input len input__ s last_acc =
  input__ `seq` -- strict in the input
  let
  new_acc = (check_accs (alex_accept `quickIndex` IBOX(s)))
  in
  new_acc `seq`
  case alexGetByte input__ of
     Nothing -> (new_acc, input__)
     Just (c, new_input) ->
#ifdef ALEX_DEBUG
      trace ("State: " ++ show IBOX(s) ++ ", char: " ++ show c) $
#endif
      case fromIntegral c of { IBOX(ord_c) ->
        let
                base   = alexIndexInt32OffAddr alex_base s
                offset = PLUS(base,ord_c)
                check  = alexIndexInt16OffAddr alex_check offset

                new_s = if GTE(offset,ILIT(0)) && EQ(check,ord_c)
                          then alexIndexInt16OffAddr alex_table offset
                          else alexIndexInt16OffAddr alex_deflt s
        in
        case new_s of
            ILIT(-1) -> (new_acc, input__)
                -- on an error, we want to keep the input *before* the
                -- character that failed, not after.
            _ -> alex_scan_tkn user__ orig_input
#ifdef ALEX_LATIN1
                   PLUS(len,ILIT(1))
                   -- issue 119: in the latin1 encoding, *each* byte is one character
#else
                   (if c < 0x80 || c >= 0xC0 then PLUS(len,ILIT(1)) else len)
                   -- note that the length is increased ONLY if this is the 1st byte in a char encoding)
#endif
                   new_input new_s new_acc
      }
  where
        check_accs (AlexAccNone) = last_acc
        check_accs (AlexAcc a  ) = AlexLastAcc a input__ IBOX(len)
        check_accs (AlexAccSkip) = AlexLastSkip  input__ IBOX(len)
#ifndef ALEX_NOPRED
        check_accs (AlexAccPred a predx rest)
           | predx user__ orig_input IBOX(len) input__
           = AlexLastAcc a input__ IBOX(len)
           | otherwise
           = check_accs rest
        check_accs (AlexAccSkipPred predx rest)
           | predx user__ orig_input IBOX(len) input__
           = AlexLastSkip input__ IBOX(len)
           | otherwise
           = check_accs rest
#endif

data AlexLastAcc
  = AlexNone
  | AlexLastAcc !Int !AlexInput !Int
  | AlexLastSkip     !AlexInput !Int

data AlexAcc user
  = AlexAccNone
  | AlexAcc Int
  | AlexAccSkip
#ifndef ALEX_NOPRED
  | AlexAccPred Int (AlexAccPred user) (AlexAcc user)
  | AlexAccSkipPred (AlexAccPred user) (AlexAcc user)

type AlexAccPred user = user -> AlexInput -> Int -> AlexInput -> Bool

-- -----------------------------------------------------------------------------
-- Predicates on a rule

alexAndPred p1 p2 user__ in1 len in2
  = p1 user__ in1 len in2 && p2 user__ in1 len in2

--alexPrevCharIsPred :: Char -> AlexAccPred _
alexPrevCharIs c _ input__ _ _ = c == alexInputPrevChar input__

alexPrevCharMatches f _ input__ _ _ = f (alexInputPrevChar input__)

--alexPrevCharIsOneOfPred :: Array Char Bool -> AlexAccPred _
alexPrevCharIsOneOf arr _ input__ _ _ = arr ! alexInputPrevChar input__

--alexRightContext :: Int -> AlexAccPred _
alexRightContext IBOX(sc) user__ _ _ input__ =
     case alex_scan_tkn user__ input__ ILIT(0) input__ sc AlexNone of
          (AlexNone, _) -> False
          _ -> True
        -- TODO: there's no need to find the longest
        -- match when checking the right context, just
        -- the first match will do.
#endif
